@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License, version 2.0, as published by the
@@ -58,7 +58,9 @@ import java.util.Map;
 
 import com.mysql.cj.Messages;
 import com.mysql.cj.MysqlType;
+import com.mysql.cj.ParseInfo;
 import com.mysql.cj.PreparedQuery;
+import com.mysql.cj.conf.PropertyDefinitions.DatabaseTerm;
 import com.mysql.cj.conf.PropertyKey;
 import com.mysql.cj.exceptions.FeatureNotAvailableException;
 import com.mysql.cj.exceptions.MysqlErrorNumbers;
@@ -70,6 +72,7 @@ import com.mysql.cj.protocol.a.result.ResultsetRowsStatic;
 import com.mysql.cj.result.DefaultColumnDefinition;
 import com.mysql.cj.result.Field;
 import com.mysql.cj.result.Row;
+import com.mysql.cj.util.SearchMode;
 import com.mysql.cj.util.StringUtils;
 import com.mysql.cj.util.Util;
 
@@ -124,7 +127,7 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
     }
 
     public class CallableStatementParamInfo implements ParameterMetaData {
-        String catalogInUse;
+        String dbInUse;
 
         boolean isFunctionCall;
 
@@ -156,7 +159,7 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
          */
         CallableStatementParamInfo(CallableStatementParamInfo fullParamInfo) {
             this.nativeSql = ((PreparedQuery<?>) CallableStatement.this.query).getOriginalSql();
-            this.catalogInUse = CallableStatement.this.getCurrentCatalog();
+            this.dbInUse = CallableStatement.this.getCurrentDatabase();
             this.isFunctionCall = fullParamInfo.isFunctionCall;
             @SuppressWarnings("synthetic-access")
             int[] localParameterMap = CallableStatement.this.placeholderToParameterIndexMap;
@@ -191,7 +194,7 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
             boolean hadRows = paramTypesRs.last();
 
             this.nativeSql = ((PreparedQuery<?>) CallableStatement.this.query).getOriginalSql();
-            this.catalogInUse = CallableStatement.this.getCurrentCatalog();
+            this.dbInUse = CallableStatement.this.getCurrentDatabase();
             this.isFunctionCall = CallableStatement.this.callingStoredFunction;
 
             if (hadRows) {
@@ -450,7 +453,7 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
      *             if an error occurs
      */
     public CallableStatement(JdbcConnection conn, CallableStatementParamInfo paramInfo) throws SQLException {
-        super(conn, paramInfo.nativeSql, paramInfo.catalogInUse);
+        super(conn, paramInfo.nativeSql, paramInfo.dbInUse);
 
         this.paramInfo = paramInfo;
         this.callingStoredFunction = this.paramInfo.isFunctionCall;
@@ -471,8 +474,8 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
      *            the connection creating this statement
      * @param sql
      *            the SQL to prepare
-     * @param catalog
-     *            the current catalog
+     * @param db
+     *            the current database
      * @param isFunctionCall
      *            is it a function call or a procedure call?
      * @return CallableStatement
@@ -480,8 +483,8 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
      *             if an error occurs
      */
 
-    protected static CallableStatement getInstance(JdbcConnection conn, String sql, String catalog, boolean isFunctionCall) throws SQLException {
-        return new CallableStatement(conn, sql, catalog, isFunctionCall);
+    protected static CallableStatement getInstance(JdbcConnection conn, String sql, String db, boolean isFunctionCall) throws SQLException {
+        return new CallableStatement(conn, sql, db, isFunctionCall);
     }
 
     /**
@@ -529,7 +532,7 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
                     int parenOpenPos = q.getOriginalSql().indexOf('(', startPos + 4);
 
                     if (parenOpenPos != -1) {
-                        int parenClosePos = StringUtils.indexOfIgnoreCase(parenOpenPos, q.getOriginalSql(), ")", "'", "'", StringUtils.SEARCH_MODE__ALL);
+                        int parenClosePos = StringUtils.indexOfIgnoreCase(parenOpenPos, q.getOriginalSql(), ")", "'", "'", SearchMode.__FULL);
 
                         if (parenClosePos != -1) {
                             List<?> parsedParameters = StringUtils.split(q.getOriginalSql().substring(parenOpenPos + 1, parenClosePos), ",", "'\"", "'\"",
@@ -564,16 +567,16 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
      *            the connection creating this statement
      * @param sql
      *            the SQL to prepare
-     * @param catalog
-     *            the current catalog
+     * @param db
+     *            the current database
      * @param isFunctionCall
      *            is it a function call or a procedure call?
      * 
      * @throws SQLException
      *             if an error occurs
      */
-    public CallableStatement(JdbcConnection conn, String sql, String catalog, boolean isFunctionCall) throws SQLException {
-        super(conn, sql, catalog);
+    public CallableStatement(JdbcConnection conn, String sql, String db, boolean isFunctionCall) throws SQLException {
+        super(conn, sql, db);
 
         this.callingStoredFunction = isFunctionCall;
 
@@ -704,8 +707,8 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
      */
     private void fakeParameterTypes(boolean isReallyProcedure) throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            String encoding = this.connection.getSession().getServerSession().getCharacterSetMetadata();
-            int collationIndex = this.connection.getSession().getServerSession().getMetadataCollationIndex();
+            String encoding = this.connection.getSession().getServerSession().getCharsetSettings().getMetadataEncoding();
+            int collationIndex = this.connection.getSession().getServerSession().getCharsetSettings().getMetadataCollationIndex();
             Field[] fields = new Field[13];
 
             fields[0] = new Field("", "PROCEDURE_CAT", collationIndex, encoding, MysqlType.CHAR, 0);
@@ -770,10 +773,10 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
                 String quotedId = this.session.getIdentifierQuoteString();
 
                 List<?> parseList = StringUtils.splitDBdotName(procName, "", quotedId, this.session.getServerSession().isNoBackslashEscapesSet());
-                String tmpCatalog = "";
+                String tmpDb = "";
                 //There *should* be 2 rows, if any.
                 if (parseList.size() == 2) {
-                    tmpCatalog = (String) parseList.get(0);
+                    tmpDb = (String) parseList.get(0);
                     procName = (String) parseList.get(1);
                 } else {
                     //keep values as they are
@@ -781,13 +784,15 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
 
                 java.sql.DatabaseMetaData dbmd = this.connection.getMetaData();
 
-                boolean useCatalog = false;
+                boolean useDb = false;
 
-                if (tmpCatalog.length() <= 0) {
-                    useCatalog = true;
+                if (tmpDb.length() <= 0) {
+                    useDb = true;
                 }
 
-                paramTypesRs = dbmd.getProcedureColumns(useCatalog ? this.getCurrentCatalog() : tmpCatalog/* null */, null, procName, "%");
+                paramTypesRs = this.session.getPropertySet().<DatabaseTerm>getEnumProperty(PropertyKey.databaseTerm).getValue() == DatabaseTerm.SCHEMA
+                        ? dbmd.getProcedureColumns(null, useDb ? this.getCurrentDatabase() : tmpDb/* null */, procName, "%")
+                        : dbmd.getProcedureColumns(useDb ? this.getCurrentDatabase() : tmpDb/* null */, null, procName, "%");
 
                 boolean hasResults = false;
                 try {
@@ -897,7 +902,8 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
     }
 
     private String extractProcedureName() throws SQLException {
-        String sanitizedSql = StringUtils.stripComments(((PreparedQuery<?>) this.query).getOriginalSql(), "`\"'", "`\"'", true, false, true, true);
+        String sanitizedSql = StringUtils.stripCommentsAndHints(((PreparedQuery<?>) this.query).getOriginalSql(), "`'\"", "`'\"",
+                !this.session.getServerSession().isNoBackslashEscapesSet());
 
         // TODO: Do this with less memory allocation
         int endCallIndex = StringUtils.indexOfIgnoreCase(sanitizedSql, "CALL ");
@@ -2278,13 +2284,13 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
             try {
                 String procName = extractProcedureName();
 
-                String catalog = this.getCurrentCatalog();
+                String db = this.getCurrentDatabase();
 
                 if (procName.indexOf(".") != -1) {
-                    catalog = procName.substring(0, procName.indexOf("."));
+                    db = procName.substring(0, procName.indexOf("."));
 
-                    if (StringUtils.startsWithIgnoreCaseAndWs(catalog, "`") && catalog.trim().endsWith("`")) {
-                        catalog = catalog.substring(1, catalog.length() - 1);
+                    if (StringUtils.startsWithIgnoreCaseAndWs(db, "`") && db.trim().endsWith("`")) {
+                        db = db.substring(1, db.length() - 1);
                     }
 
                     procName = procName.substring(procName.indexOf(".") + 1);
@@ -2294,7 +2300,7 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
                 ps.setMaxRows(0);
                 ps.setFetchSize(0);
 
-                ps.setString(1, catalog);
+                ps.setString(1, db);
                 ps.setString(2, procName);
                 rs = ps.executeQuery();
                 if (rs.next()) {
@@ -2327,7 +2333,20 @@ public class CallableStatement extends ClientPreparedStatement implements java.s
 
     @Override
     protected boolean checkReadOnlySafeStatement() throws SQLException {
-        return (super.checkReadOnlySafeStatement() || this.checkReadOnlyProcedure());
+        if (ParseInfo.isReadOnlySafeQuery(((PreparedQuery<?>) this.query).getOriginalSql(), this.session.getServerSession().isNoBackslashEscapesSet())) {
+            String sql = ((PreparedQuery<?>) this.query).getOriginalSql();
+            int statementKeywordPos = ParseInfo.indexOfStatementKeyword(sql, this.session.getServerSession().isNoBackslashEscapesSet());
+            if (StringUtils.startsWithIgnoreCaseAndWs(sql, "CALL", statementKeywordPos)
+                    || StringUtils.startsWithIgnoreCaseAndWs(sql, "SELECT", statementKeywordPos)) {
+                // "CALL" and "SELECT" are read-only safe but the routine they call may not be.
+                if (!this.connection.isReadOnly()) { // Only proceed with checking the routine body if really needed.
+                    return true;
+                }
+                return checkReadOnlyProcedure();
+            }
+            return true;
+        }
+        return !this.connection.isReadOnly();
     }
 
     @Override
