@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License, version 2.0, as published by the
@@ -54,16 +54,16 @@ import java.sql.SQLXML;
 import java.sql.Struct;
 import java.sql.Time;
 import java.sql.Timestamp;
+import java.time.Duration;
 import java.time.LocalDate;
 import java.time.LocalDateTime;
 import java.time.LocalTime;
 import java.time.OffsetDateTime;
 import java.time.OffsetTime;
-import java.time.format.DateTimeParseException;
+import java.time.ZonedDateTime;
 import java.util.Calendar;
 import java.util.HashSet;
 import java.util.Set;
-import java.util.TimeZone;
 
 import com.mysql.cj.Messages;
 import com.mysql.cj.MysqlType;
@@ -100,6 +100,7 @@ import com.mysql.cj.result.BinaryStreamValueFactory;
 import com.mysql.cj.result.BooleanValueFactory;
 import com.mysql.cj.result.ByteValueFactory;
 import com.mysql.cj.result.DoubleValueFactory;
+import com.mysql.cj.result.DurationValueFactory;
 import com.mysql.cj.result.Field;
 import com.mysql.cj.result.FloatValueFactory;
 import com.mysql.cj.result.IntegerValueFactory;
@@ -107,12 +108,16 @@ import com.mysql.cj.result.LocalDateTimeValueFactory;
 import com.mysql.cj.result.LocalDateValueFactory;
 import com.mysql.cj.result.LocalTimeValueFactory;
 import com.mysql.cj.result.LongValueFactory;
+import com.mysql.cj.result.OffsetDateTimeValueFactory;
+import com.mysql.cj.result.OffsetTimeValueFactory;
 import com.mysql.cj.result.ShortValueFactory;
 import com.mysql.cj.result.SqlDateValueFactory;
 import com.mysql.cj.result.SqlTimeValueFactory;
 import com.mysql.cj.result.SqlTimestampValueFactory;
 import com.mysql.cj.result.StringValueFactory;
+import com.mysql.cj.result.UtilCalendarValueFactory;
 import com.mysql.cj.result.ValueFactory;
+import com.mysql.cj.result.ZonedDateTimeValueFactory;
 import com.mysql.cj.util.LogUtils;
 import com.mysql.cj.util.StringUtils;
 
@@ -174,6 +179,9 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     protected boolean useUsageAdvisor = false;
     protected boolean gatherPerfMetrics = false;
 
+    /** Is ResultSet.TYPE_FORWARD_ONLY scroll tolerant? */
+    protected boolean scrollTolerant = false;
+
     /** The warning chain */
     protected java.sql.SQLWarning warningChain = null;
 
@@ -197,10 +205,16 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     private ValueFactory<Time> defaultTimeValueFactory;
     private ValueFactory<Timestamp> defaultTimestampValueFactory;
 
+    private ValueFactory<Calendar> defaultUtilCalendarValueFactory;
+
     private ValueFactory<LocalDate> defaultLocalDateValueFactory;
     private ValueFactory<LocalDateTime> defaultLocalDateTimeValueFactory;
     private ValueFactory<LocalTime> defaultLocalTimeValueFactory;
 
+    private ValueFactory<OffsetTime> defaultOffsetTimeValueFactory;
+    private ValueFactory<OffsetDateTime> defaultOffsetDateTimeValueFactory;
+    private ValueFactory<ZonedDateTime> defaultZonedDateTimeValueFactory;
+
     protected RuntimeProperty<Boolean> emulateLocators;
     protected boolean yearIsDateType = true;
 
@@ -256,6 +270,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
         this.yearIsDateType = pset.getBooleanProperty(PropertyKey.yearIsDateType).getValue();
         this.useUsageAdvisor = pset.getBooleanProperty(PropertyKey.useUsageAdvisor).getValue();
         this.gatherPerfMetrics = pset.getBooleanProperty(PropertyKey.gatherPerfMetrics).getValue();
+        this.scrollTolerant = pset.getBooleanProperty(PropertyKey.scrollTolerantForwardOnly).getValue();
 
         this.booleanValueFactory = new BooleanValueFactory(pset);
         this.byteValueFactory = new ByteValueFactory(pset);
@@ -267,13 +282,23 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
         this.bigDecimalValueFactory = new BigDecimalValueFactory(pset);
         this.binaryStreamValueFactory = new BinaryStreamValueFactory(pset);
 
-        this.defaultTimeValueFactory = new SqlTimeValueFactory(pset, null, this.session.getServerSession().getServerTimeZone(), this);
-        this.defaultTimestampValueFactory = new SqlTimestampValueFactory(pset, null, this.session.getServerSession().getServerTimeZone());
+        this.defaultTimeValueFactory = new SqlTimeValueFactory(pset, null, this.session.getServerSession().getDefaultTimeZone(), this);
+        this.defaultTimestampValueFactory = new SqlTimestampValueFactory(pset, null, this.session.getServerSession().getDefaultTimeZone(),
+                this.session.getServerSession().getSessionTimeZone());
+
+        this.defaultUtilCalendarValueFactory = new UtilCalendarValueFactory(pset, this.session.getServerSession().getDefaultTimeZone(),
+                this.session.getServerSession().getSessionTimeZone());
 
         this.defaultLocalDateValueFactory = new LocalDateValueFactory(pset, this);
         this.defaultLocalTimeValueFactory = new LocalTimeValueFactory(pset, this);
         this.defaultLocalDateTimeValueFactory = new LocalDateTimeValueFactory(pset);
 
+        this.defaultOffsetTimeValueFactory = new OffsetTimeValueFactory(pset, this.session.getProtocol().getServerSession().getDefaultTimeZone());
+        this.defaultOffsetDateTimeValueFactory = new OffsetDateTimeValueFactory(pset, this.session.getProtocol().getServerSession().getDefaultTimeZone(),
+                this.session.getProtocol().getServerSession().getSessionTimeZone());
+        this.defaultZonedDateTimeValueFactory = new ZonedDateTimeValueFactory(pset, this.session.getProtocol().getServerSession().getDefaultTimeZone(),
+                this.session.getProtocol().getServerSession().getSessionTimeZone());
+
         this.columnDefinition = tuples.getMetadata();
         this.rowData = tuples;
         this.updateCount = this.rowData.size();
@@ -346,7 +371,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     @Override
     public boolean absolute(int row) throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY) {
+            if (isStrictlyForwardOnly()) {
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.ForwardOnly"));
             }
 
@@ -394,7 +419,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     @Override
     public void afterLast() throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY) {
+            if (isStrictlyForwardOnly()) {
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.ForwardOnly"));
             }
 
@@ -410,7 +435,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     @Override
     public void beforeFirst() throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY) {
+            if (isStrictlyForwardOnly()) {
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.ForwardOnly"));
             }
 
@@ -565,7 +590,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     @Override
     public boolean first() throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY) {
+            if (isStrictlyForwardOnly()) {
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.ForwardOnly"));
             }
 
@@ -850,7 +875,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
         String stringVal = this.thisRow.getValue(columnIndex - 1, vf);
 
         if (this.padCharsWithSpace && stringVal != null && f.getMysqlTypeId() == MysqlType.FIELD_TYPE_STRING) {
-            int maxBytesPerChar = this.session.getServerSession().getMaxBytesPerChar(f.getCollationIndex(), f.getEncoding());
+            int maxBytesPerChar = this.session.getServerSession().getCharsetSettings().getMaxBytesPerChar(f.getCollationIndex(), f.getEncoding());
             int fieldLength = (int) f.getLength() /* safe, bytes in a CHAR <= 1024 */ / maxBytesPerChar; /* safe, this will never be 0 */
             return StringUtils.padString(stringVal, fieldLength);
         }
@@ -893,7 +918,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
         checkRowPos();
         checkColumnBounds(columnIndex);
         ValueFactory<Time> vf = new SqlTimeValueFactory(this.session.getPropertySet(), cal,
-                cal != null ? cal.getTimeZone() : this.session.getServerSession().getServerTimeZone());
+                cal != null ? cal.getTimeZone() : this.session.getServerSession().getDefaultTimeZone());
         return this.thisRow.getValue(columnIndex - 1, vf);
     }
 
@@ -932,25 +957,18 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
         return this.thisRow.getValue(columnIndex - 1, this.defaultLocalTimeValueFactory);
     }
 
-    /*
-     * This method is optimized by saving the configuration for the last-used cal/tz. If it's re-used, we don't need to create a new value factory (and thus
-     * calendar, etc) instance
-     */
-    private TimeZone lastTsCustomTz;
-    private ValueFactory<Timestamp> customTsVf;
+    public Calendar getUtilCalendar(int columnIndex) throws SQLException {
+        checkRowPos();
+        checkColumnBounds(columnIndex);
+        return this.thisRow.getValue(columnIndex - 1, this.defaultUtilCalendarValueFactory);
+    }
 
     @Override
     public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException {
         checkRowPos();
         checkColumnBounds(columnIndex);
-
-        TimeZone tz = cal != null ? cal.getTimeZone() : this.session.getServerSession().getServerTimeZone();
-        if (this.customTsVf != null && tz == this.lastTsCustomTz) {
-            return this.thisRow.getValue(columnIndex - 1, this.customTsVf);
-        }
-        ValueFactory<Timestamp> vf = new SqlTimestampValueFactory(this.session.getPropertySet(), cal, tz);
-        this.lastTsCustomTz = tz;
-        this.customTsVf = vf;
+        ValueFactory<Timestamp> vf = new SqlTimestampValueFactory(this.session.getPropertySet(), cal, this.session.getServerSession().getDefaultTimeZone(),
+                this.session.getServerSession().getSessionTimeZone());
         return this.thisRow.getValue(columnIndex - 1, vf);
     }
 
@@ -1250,9 +1268,11 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
                 return getTime(columnIndex);
 
             case TIMESTAMP:
-            case DATETIME:
                 return getTimestamp(columnIndex);
 
+            case DATETIME:
+                return getLocalDateTime(columnIndex);
+
             default:
                 return getString(columnIndex);
         }
@@ -1322,6 +1342,13 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
             } else if (type.equals(Timestamp.class)) {
                 return (T) getTimestamp(columnIndex);
 
+            } else if (type.equals(java.util.Date.class)) {
+                Timestamp ts = getTimestamp(columnIndex);
+                return ts == null ? null : (T) java.util.Date.from(ts.toInstant());
+
+            } else if (type.equals(java.util.Calendar.class)) {
+                return (T) getUtilCalendar(columnIndex);
+
             } else if (type.equals(Clob.class)) {
                 return (T) getClob(columnIndex);
 
@@ -1359,21 +1386,24 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
                 return (T) getLocalTime(columnIndex);
 
             } else if (type.equals(OffsetDateTime.class)) {
-                try {
-                    String odt = getString(columnIndex);
-                    return odt == null ? null : (T) OffsetDateTime.parse(odt);
-                } catch (DateTimeParseException e) {
-                    // Let it continue and try by object deserialization.
-                }
+                checkRowPos();
+                checkColumnBounds(columnIndex);
+                return (T) this.thisRow.getValue(columnIndex - 1, this.defaultOffsetDateTimeValueFactory);
 
             } else if (type.equals(OffsetTime.class)) {
-                try {
-                    String ot = getString(columnIndex);
-                    return ot == null ? null : (T) OffsetTime.parse(getString(columnIndex));
-                } catch (DateTimeParseException e) {
-                    // Let it continue and try by object deserialization.
-                }
+                checkRowPos();
+                checkColumnBounds(columnIndex);
+                return (T) this.thisRow.getValue(columnIndex - 1, this.defaultOffsetTimeValueFactory);
 
+            } else if (type.equals(ZonedDateTime.class)) {
+                checkRowPos();
+                checkColumnBounds(columnIndex);
+                return (T) this.thisRow.getValue(columnIndex - 1, this.defaultZonedDateTimeValueFactory);
+
+            } else if (type.equals(Duration.class)) {
+                checkRowPos();
+                checkColumnBounds(columnIndex);
+                return (T) this.thisRow.getValue(columnIndex - 1, new DurationValueFactory(this.session.getPropertySet()));
             }
 
             if (this.connection.getPropertySet().getBooleanProperty(PropertyKey.autoDeserialize).getValue()) {
@@ -1689,10 +1719,21 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
         }
     }
 
+    /**
+     * Checks whether this ResultSet is scrollable even if its type is ResultSet.TYPE_FORWARD_ONLY. Required for backwards compatibility.
+     * 
+     * @return
+     *         <code>true</code> if this result set type is ResultSet.TYPE_FORWARD_ONLY and the connection property 'scrollTolerantForwardOnly' has not been set
+     *         to <code>true</code>.
+     */
+    protected boolean isStrictlyForwardOnly() {
+        return this.resultSetType == ResultSet.TYPE_FORWARD_ONLY && !this.scrollTolerant;
+    }
+
     @Override
     public boolean last() throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY) {
+            if (isStrictlyForwardOnly()) {
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.ForwardOnly"));
             }
 
@@ -1798,7 +1839,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     @Override
     public boolean previous() throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY) {
+            if (isStrictlyForwardOnly()) {
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.ForwardOnly"));
             }
 
@@ -1920,7 +1961,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
     @Override
     public boolean relative(int rows) throws SQLException {
         synchronized (checkClosed().getConnectionMutex()) {
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY) {
+            if (isStrictlyForwardOnly()) {
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.ForwardOnly"));
             }
 
@@ -1962,7 +2003,7 @@ public class ResultSetImpl extends NativeResultset implements ResultSetInternalM
                         MysqlErrorNumbers.SQL_STATE_ILLEGAL_ARGUMENT, getExceptionInterceptor());
             }
 
-            if (getType() == ResultSet.TYPE_FORWARD_ONLY && direction != FETCH_FORWARD) {
+            if (isStrictlyForwardOnly() && direction != FETCH_FORWARD) {
                 String constName = direction == ResultSet.FETCH_REVERSE ? "ResultSet.FETCH_REVERSE" : "ResultSet.FETCH_UNKNOWN";
                 throw ExceptionFactory.createException(Messages.getString("ResultSet.Unacceptable_value_for_fetch_direction", new Object[] { constName }));
             }
