@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License, version 2.0, as published by the
@@ -30,6 +30,13 @@
 package testsuite;
 
 import static com.mysql.cj.util.StringUtils.isNullOrEmpty;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 
 import java.io.BufferedOutputStream;
 import java.io.File;
@@ -37,6 +44,8 @@ import java.io.FileOutputStream;
 import java.io.FilenameFilter;
 import java.io.IOException;
 import java.lang.reflect.Method;
+import java.net.InetAddress;
+import java.net.UnknownHostException;
 import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.PreparedStatement;
@@ -53,6 +62,10 @@ import java.util.Set;
 import java.util.StringJoiner;
 import java.util.concurrent.Callable;
 
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.TestInfo;
+
 import com.mysql.cj.ServerVersion;
 import com.mysql.cj.conf.ConnectionUrl;
 import com.mysql.cj.conf.ConnectionUrlParser;
@@ -65,33 +78,18 @@ import com.mysql.cj.jdbc.ha.ReplicationConnection;
 import com.mysql.cj.util.StringUtils;
 import com.mysql.cj.util.Util;
 
-import junit.framework.TestCase;
-
 /**
  * Base class for all test cases. Creates connections, statements, etc. and closes them.
  */
-public abstract class BaseTestCase extends TestCase {
-
-    // next variables disable some tests
-    protected boolean DISABLED_testBug15121 = true; // TODO needs to be fixed on server
-    protected boolean DISABLED_testBug7033 = true; // TODO disabled for unknown reason
-    protected boolean DISABLED_testBug2654 = true; // TODO check if it's still a server-level bug
-    protected boolean DISABLED_testBug5136 = true; // TODO disabled for unknown reason
-    protected boolean DISABLED_testBug65503 = true; // TODO disabled for unknown reason
-    protected boolean DISABLED_testContention = true; // TODO disabled for unknown reason
-    protected boolean DISABLED_testBug3620new = true; // TODO this test is working in c/J 5.1 but fails here; disabled for later analysis
+public abstract class BaseTestCase {
 
     /**
      * JDBC URL, initialized from com.mysql.cj.testsuite.url system property, or defaults to jdbc:mysql:///test and its connection URL.
      */
     public static String dbUrl = "jdbc:mysql:///test";
+    public static String timeZoneFreeDbUrl = "jdbc:mysql:///test";
     protected static ConnectionUrl mainConnectionUrl = null;
-
-    /**
-     * JDBC URL, initialized from com.mysql.cj.testsuite.url.openssl system property and its connection URL
-     */
-    protected static String sha256Url = null;
-    protected static ConnectionUrl sha256ConnectionUrl = null;
+    protected boolean isOpenSSL = false;
 
     /** Instance counter */
     private static int instanceCount = 1;
@@ -99,8 +97,6 @@ public abstract class BaseTestCase extends TestCase {
     /** Connection to server, initialized in setUp() Cleaned up in tearDown(). */
     protected Connection conn = null;
 
-    protected Connection sha256Conn = null;
-
     /** Server version `this.conn' is connected to. */
     protected ServerVersion serverVersion;
 
@@ -113,10 +109,13 @@ public abstract class BaseTestCase extends TestCase {
     /** My instance number */
     private int myInstanceNumber = 0;
 
+    /** Is MySQL running locally? */
+    private Boolean mysqlRunningLocally = null;
+
     /**
      * Default catalog.
      */
-    protected final String dbName;
+    protected String dbName;
 
     /**
      * PreparedStatement to be used in tests, not initialized. Cleaned up in
@@ -129,42 +128,49 @@ public abstract class BaseTestCase extends TestCase {
      */
     protected ResultSet rs = null;
 
-    protected ResultSet sha256Rs = null;
-
     /**
      * Statement to be used in tests, initialized in setUp(). Cleaned up in
      * tearDown().
      */
     protected Statement stmt = null;
 
-    protected Statement sha256Stmt = null;
-
     private boolean isOnCSFS = true;
 
     /**
      * Creates a new BaseTestCase object.
-     * 
-     * @param name
-     *            The name of the JUnit test case
      */
-    public BaseTestCase(String name) {
-        super(name);
+    public BaseTestCase() {
         this.myInstanceNumber = instanceCount++;
 
         String newDbUrl = System.getProperty(PropertyDefinitions.SYSP_testsuite_url);
 
         if ((newDbUrl != null) && (newDbUrl.trim().length() != 0)) {
-            dbUrl = newDbUrl;
+            dbUrl = sanitizeDbName(newDbUrl);
         }
         mainConnectionUrl = ConnectionUrl.getConnectionUrlInstance(dbUrl, null);
         this.dbName = mainConnectionUrl.getDatabase();
 
-        String defaultSha256Url = System.getProperty(PropertyDefinitions.SYSP_testsuite_url_openssl);
+        timeZoneFreeDbUrl = dbUrl.replaceAll(PropertyKey.connectionTimeZone.getKeyName() + "=", PropertyKey.connectionTimeZone.getKeyName() + "VOID=")
+                .replaceAll("serverTimezone=", "serverTimezoneVOID=");
+    }
 
-        if ((defaultSha256Url != null) && (defaultSha256Url.trim().length() != 0)) {
-            sha256Url = defaultSha256Url;
-            sha256ConnectionUrl = ConnectionUrl.getConnectionUrlInstance(sha256Url, null);
+    private String sanitizeDbName(String url) {
+        ConnectionUrl parsedUrl = ConnectionUrl.getConnectionUrlInstance(url, null);
+        if (StringUtils.isNullOrEmpty(parsedUrl.getDatabase())) {
+            List<String> splitUp = StringUtils.split(url, "\\?", true);
+            StringBuilder value = new StringBuilder();
+            for (int i = 0; i < splitUp.size(); i++) {
+                value.append(splitUp.get(i));
+                if (i == 0) {
+                    if (!splitUp.get(i).endsWith("/")) {
+                        value.append("/");
+                    }
+                    value.append("cjtest_8_0?");
+                }
+            }
+            url = value.toString();
         }
+        return url;
     }
 
     protected void createSchemaObject(String objectType, String objectName, String columnsAndOtherStuff) throws SQLException {
@@ -323,19 +329,6 @@ public abstract class BaseTestCase extends TestCase {
         }
     }
 
-    protected Connection getAdminConnection() throws SQLException {
-        return getAdminConnectionWithProps(new Properties());
-    }
-
-    protected Connection getAdminConnectionWithProps(Properties props) throws SQLException {
-        String adminUrl = System.getProperty(PropertyDefinitions.SYSP_testsuite_url_admin);
-
-        if (adminUrl != null) {
-            return DriverManager.getConnection(adminUrl, props);
-        }
-        return null;
-    }
-
     public Connection getConnectionWithProps(String propsList) throws SQLException {
         return getConnectionWithProps(dbUrl, propsList);
     }
@@ -386,16 +379,10 @@ public abstract class BaseTestCase extends TestCase {
     }
 
     protected Connection getNewConnection() throws SQLException {
-        return DriverManager.getConnection(dbUrl);
-    }
-
-    protected Connection getNewSha256Connection() throws SQLException {
-        if (sha256Url != null) {
-            Properties props = new Properties();
-            props.setProperty(PropertyKey.allowPublicKeyRetrieval.getKeyName(), "true");
-            return DriverManager.getConnection(sha256Url, props);
-        }
-        return null;
+        Properties props = new Properties();
+        props.setProperty(PropertyKey.sslMode.getKeyName(), "DISABLED");
+        props.setProperty(PropertyKey.allowPublicKeyRetrieval.getKeyName(), "true");
+        return DriverManager.getConnection(dbUrl, props);
     }
 
     /**
@@ -416,7 +403,6 @@ public abstract class BaseTestCase extends TestCase {
         }
 
         return null;
-
     }
 
     /**
@@ -471,7 +457,7 @@ public abstract class BaseTestCase extends TestCase {
 
     /**
      * Some tests build connections strings for internal usage but, in order for them to work, they may require some connection properties set in the main test
-     * suite URL. For example 'serverTimezone' is one of those properties.
+     * suite URL. For example 'connectionTimeZone' is one of those properties.
      * 
      * @param props
      *            the Properties object where to add the missing connection properties
@@ -483,8 +469,8 @@ public abstract class BaseTestCase extends TestCase {
             props = new Properties();
         }
 
-        // Add 'serverTimezone' if set in test suite URL and missing from props.
-        String propKey = PropertyKey.serverTimezone.getKeyName();
+        // Add 'connectionTimeZone' if set in test suite URL and missing from props.
+        String propKey = PropertyKey.connectionTimeZone.getKeyName();
         String origTzValue = null;
         if (!props.containsKey(propKey) && (origTzValue = mainConnectionUrl.getOriginalProperties().get(propKey)) != null) {
             props.setProperty(propKey, origTzValue);
@@ -589,10 +575,6 @@ public abstract class BaseTestCase extends TestCase {
         return getSingleIndexedValueWithQuery(1, query);
     }
 
-    protected boolean isAdminConnectionConfigured() {
-        return System.getProperty(PropertyDefinitions.SYSP_testsuite_url_admin) != null;
-    }
-
     protected boolean isServerRunningOnWindows() throws SQLException {
         return (getMysqlVariable("datadir").indexOf('\\') != -1);
     }
@@ -621,7 +603,7 @@ public abstract class BaseTestCase extends TestCase {
     }
 
     protected final boolean runLongTests() {
-        return runTestIfSysPropDefined(PropertyDefinitions.SYSP_testsuite_runLongTests);
+        return isSysPropDefined(PropertyDefinitions.SYSP_testsuite_runLongTests);
     }
 
     /**
@@ -633,36 +615,36 @@ public abstract class BaseTestCase extends TestCase {
      * 
      * @return true if the property is defined.
      */
-    protected boolean runTestIfSysPropDefined(String propName) {
+    protected boolean isSysPropDefined(String propName) {
         String prop = System.getProperty(propName);
-
         return (prop != null) && (prop.length() > 0);
     }
 
-    protected boolean runMultiHostTests() {
-        return !runTestIfSysPropDefined(PropertyDefinitions.SYSP_testsuite_disable_multihost_tests);
-    }
-
     /**
      * Creates resources used by all tests.
      * 
+     * @param testInfo
+     * 
      * @throws Exception
      *             if an error occurs.
      */
-    @Override
-    public void setUp() throws Exception {
-        System.out.println("Running test " + getClass().getName() + "#" + getName());
-        System.out.println("################################################################################");
-        Class.forName(this.dbClass).newInstance();
+    @BeforeEach
+    public void setUpBase(TestInfo testInfo) throws Exception {
+        System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
+        System.out.println("Running test " + testInfo.getTestClass().get().getName() + "#" + testInfo.getDisplayName());
+        System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
+        Class.forName(this.dbClass);
         this.createdObjects = new ArrayList<>();
 
         Properties props = new Properties();
         props.setProperty(PropertyKey.useSSL.getKeyName(), "false"); // testsuite is built upon non-SSL default connection
         props.setProperty(PropertyKey.allowPublicKeyRetrieval.getKeyName(), "true");
+        props.setProperty(PropertyKey.createDatabaseIfNotExist.getKeyName(), "true");
+        if (StringUtils.isNullOrEmpty(mainConnectionUrl.getDatabase())) {
+            props.setProperty(PropertyKey.DBNAME.getKeyName(), this.dbName);
+        }
         this.conn = DriverManager.getConnection(dbUrl, props);
 
-        this.sha256Conn = sha256Url == null ? null : DriverManager.getConnection(sha256Url, props);
-
         this.serverVersion = ((JdbcConnection) this.conn).getServerVersion();
 
         this.stmt = this.conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
@@ -672,6 +654,25 @@ public abstract class BaseTestCase extends TestCase {
                 this.rs = this.stmt.executeQuery("SELECT VERSION()");
                 this.rs.next();
                 logDebug("Connected to " + this.rs.getString(1));
+                this.rs.close();
+                this.rs = this.stmt.executeQuery("SHOW VARIABLES LIKE 'auto_generate_certs'");
+                if (this.rs.next()) {
+                    this.isOpenSSL = true;
+                }
+
+                // ensure max_connections value is enough to run tests
+                this.rs.close();
+                this.rs = this.stmt.executeQuery("SHOW VARIABLES LIKE 'max_connections'");
+                this.rs.next();
+                int maxConnections = this.rs.getInt(2);
+
+                this.rs = this.stmt.executeQuery("show status like 'threads_connected'");
+                this.rs.next();
+                int usedConnections = this.rs.getInt(2);
+
+                if (maxConnections - usedConnections < 200) {
+                    this.stmt.executeUpdate("SET GLOBAL max_connections=" + (maxConnections + 200));
+                }
             } else {
                 logDebug("Connected to " + this.conn.getMetaData().getDatabaseProductName() + " / " + this.conn.getMetaData().getDatabaseProductVersion());
             }
@@ -683,34 +684,15 @@ public abstract class BaseTestCase extends TestCase {
         }
 
         this.isOnCSFS = !this.conn.getMetaData().storesLowerCaseIdentifiers();
-
-        if (this.sha256Conn != null) {
-            this.sha256Stmt = this.sha256Conn.createStatement();
-
-            try {
-                if (sha256Url.indexOf("mysql") != -1) {
-                    this.sha256Rs = this.sha256Stmt.executeQuery("SELECT VERSION()");
-                    this.sha256Rs.next();
-                    logDebug("Connected to " + this.sha256Rs.getString(1));
-                } else {
-                    logDebug("Connected to " + this.sha256Conn.getMetaData().getDatabaseProductName() + " / "
-                            + this.sha256Conn.getMetaData().getDatabaseProductVersion());
-                }
-            } finally {
-                if (this.sha256Rs != null) {
-                    this.sha256Rs.close();
-                    this.sha256Rs = null;
-                }
-            }
-        }
-
     }
 
     /**
      * Destroys resources created during the test case.
+     * 
+     * @throws Exception
      */
-    @Override
-    public void tearDown() throws Exception {
+    @AfterEach
+    public void tearDownBase() throws Exception {
         if (this.rs != null) {
             try {
                 this.rs.close();
@@ -718,22 +700,8 @@ public abstract class BaseTestCase extends TestCase {
             }
         }
 
-        if (this.sha256Rs != null) {
-            try {
-                this.sha256Rs.close();
-            } catch (SQLException SQLE) {
-            }
-        }
-
         if (System.getProperty(PropertyDefinitions.SYSP_testsuite_retainArtifacts) == null) {
-            Statement st = this.conn == null || this.conn.isClosed() ? getNewConnection().createStatement() : this.conn.createStatement();
-            Statement sha256st;
-            if (this.sha256Conn == null || this.sha256Conn.isClosed()) {
-                Connection c = getNewSha256Connection();
-                sha256st = c == null ? null : c.createStatement();
-            } else {
-                sha256st = this.sha256Conn.createStatement();
-            }
+            Statement st = (this.conn == null || this.conn.isClosed() ? getNewConnection() : this.conn).createStatement();
 
             for (int i = 0; i < this.createdObjects.size(); i++) {
                 String[] objectInfo = this.createdObjects.get(i);
@@ -742,16 +710,8 @@ public abstract class BaseTestCase extends TestCase {
                     dropSchemaObject(st, objectInfo[0], objectInfo[1]);
                 } catch (SQLException SQLE) {
                 }
-
-                try {
-                    dropSchemaObject(sha256st, objectInfo[0], objectInfo[1]);
-                } catch (SQLException SQLE) {
-                }
             }
             st.close();
-            if (sha256st != null) {
-                sha256st.close();
-            }
         }
 
         if (this.stmt != null) {
@@ -761,13 +721,6 @@ public abstract class BaseTestCase extends TestCase {
             }
         }
 
-        if (this.sha256Stmt != null) {
-            try {
-                this.sha256Stmt.close();
-            } catch (SQLException SQLE) {
-            }
-        }
-
         if (this.pstmt != null) {
             try {
                 this.pstmt.close();
@@ -781,13 +734,6 @@ public abstract class BaseTestCase extends TestCase {
             } catch (SQLException SQLE) {
             }
         }
-
-        if (this.sha256Conn != null) {
-            try {
-                this.sha256Conn.close();
-            } catch (SQLException SQLE) {
-            }
-        }
     }
 
     /**
@@ -816,6 +762,8 @@ public abstract class BaseTestCase extends TestCase {
      *            the major version to meet
      * @param minor
      *            the minor version to meet
+     * @param subminor
+     *            the subminor version to meet
      * 
      * @return boolean if the major/minor is met
      * 
@@ -828,6 +776,8 @@ public abstract class BaseTestCase extends TestCase {
 
     /**
      * Checks whether the server we're connected to is a MySQL Community edition
+     * 
+     * @return true if connected to a community/gpl server
      */
     protected boolean isCommunityEdition() {
         return Util.isCommunityEdition(((JdbcConnection) this.conn).getServerVersion().toString());
@@ -835,6 +785,8 @@ public abstract class BaseTestCase extends TestCase {
 
     /**
      * Checks whether the server we're connected to is an MySQL Enterprise edition
+     * 
+     * @return true if connected to an enterprise/commercial server
      */
     protected boolean isEnterpriseEdition() {
         return Util.isEnterpriseEdition(((JdbcConnection) this.conn).getServerVersion().toString());
@@ -855,6 +807,24 @@ public abstract class BaseTestCase extends TestCase {
         return (vmVendor != null && vmVendor.toUpperCase(Locale.US).startsWith("BEA"));
     }
 
+    protected boolean isMysqlRunningLocally() {
+        if (this.mysqlRunningLocally != null) {
+            return this.mysqlRunningLocally;
+        }
+        try {
+            String clientHostname = InetAddress.getLocalHost().getHostName();
+
+            this.rs = this.stmt.executeQuery("SHOW VARIABLES LIKE 'hostname'");
+            this.rs.next();
+            String serverHostname = this.rs.getString(2);
+
+            this.mysqlRunningLocally = clientHostname.equalsIgnoreCase(serverHostname);
+        } catch (UnknownHostException | SQLException e) {
+            this.mysqlRunningLocally = false;
+        }
+        return this.mysqlRunningLocally;
+    }
+
     protected String randomString() {
         int length = (int) (Math.random() * 32);
 
@@ -892,7 +862,7 @@ public abstract class BaseTestCase extends TestCase {
     protected void assertResultSetLength(ResultSet rset, int len) throws Exception {
         int oldRowPos = rset.getRow();
         rset.last();
-        assertEquals("Result set length", len, rset.getRow());
+        assertEquals(len, rset.getRow(), "Result set length");
         if (oldRowPos > 0) {
             rset.absolute(oldRowPos);
         } else {
@@ -920,19 +890,19 @@ public abstract class BaseTestCase extends TestCase {
                 rsAsString.append(", ");
 
                 if (controlObj == null) {
-                    assertNull("Expected null, see last row: \n" + rsAsString.toString(), testObj);
+                    assertNull(testObj, "Expected null, see last row: \n" + rsAsString.toString());
                 } else {
-                    assertNotNull("Expected non-null, see last row: \n" + rsAsString.toString(), testObj);
+                    assertNotNull(testObj, "Expected non-null, see last row: \n" + rsAsString.toString());
                 }
 
                 if (controlObj instanceof Float) {
-                    assertEquals("Float comparison failed, see last row: \n" + rsAsString.toString(), ((Float) controlObj).floatValue(),
-                            ((Float) testObj).floatValue(), 0.1);
+                    assertEquals(((Float) controlObj).floatValue(), ((Float) testObj).floatValue(), 0.1,
+                            "Float comparison failed, see last row: \n" + rsAsString.toString());
                 } else if (controlObj instanceof Double) {
-                    assertEquals("Double comparison failed, see last row: \n" + rsAsString.toString(), ((Double) controlObj).doubleValue(),
-                            ((Double) testObj).doubleValue(), 0.1);
+                    assertEquals(((Double) controlObj).doubleValue(), ((Double) testObj).doubleValue(), 0.1,
+                            "Double comparison failed, see last row: \n" + rsAsString.toString());
                 } else {
-                    assertEquals("Value comparison failed, see last row: \n" + rsAsString.toString(), controlObj, testObj);
+                    assertEquals(controlObj, testObj, "Value comparison failed, see last row: \n" + rsAsString.toString());
                 }
             }
         }
@@ -949,7 +919,7 @@ public abstract class BaseTestCase extends TestCase {
             }
         }
 
-        assertTrue("Found " + howMuchMore + " extra rows in result set to be compared: ", howMuchMore == 0);
+        assertTrue(howMuchMore == 0, "Found " + howMuchMore + " extra rows in result set to be compared: ");
     }
 
     protected static <EX extends Throwable> EX assertThrows(Class<EX> throwable, Callable<?> testRoutine) {
@@ -971,15 +941,10 @@ public abstract class BaseTestCase extends TestCase {
         try {
             testRoutine.call();
         } catch (Throwable t) {
-            if (!throwable.isAssignableFrom(t.getClass())) {
-                fail(message + "expected exception of type '" + throwable.getName() + "' but instead a exception of type '" + t.getClass().getName()
-                        + "' was thrown.");
-            }
-
-            if (msgMatchesRegex != null && !t.getMessage().matches(msgMatchesRegex)) {
-                fail(message + "the error message «" + t.getMessage() + "» was expected to match «" + msgMatchesRegex + "».");
-            }
-
+            assertTrue(throwable.isAssignableFrom(t.getClass()), message + "expected exception of type '" + throwable.getName()
+                    + "' but instead an exception of type '" + t.getClass().getName() + "' was thrown.");
+            assertFalse(msgMatchesRegex != null && !t.getMessage().matches(msgMatchesRegex),
+                    message + "the error message «" + t.getMessage() + "» was expected to match «" + msgMatchesRegex + "».");
             return throwable.cast(t);
         }
         fail(message + "expected exception of type '" + throwable.getName() + "'.");
@@ -989,9 +954,9 @@ public abstract class BaseTestCase extends TestCase {
     }
 
     protected void assertByteArrayEquals(String message, byte[] expected, byte[] actual) {
-        assertEquals(message + " - array lenght", expected.length, actual.length);
+        assertEquals(expected.length, actual.length, message + " - array lenght");
         for (int i = 0, s = expected.length; i < s; i++) {
-            assertEquals(message + " - element at " + i, expected[i], actual[i]);
+            assertEquals(expected[i], actual[i], message + " - element at " + i);
         }
     }
 
@@ -1021,7 +986,22 @@ public abstract class BaseTestCase extends TestCase {
             delimiter = " ~ ";
         }
 
-        assertEquals("Connections history", expectedHist.toString(), actualHist.toString());
+        assertEquals(expectedHist.toString(), actualHist.toString(), "Connections history");
+    }
+
+    protected static void assertSecureConnection(Connection conn) throws Exception {
+        try (Statement stmt = conn.createStatement(); ResultSet rs = stmt.executeQuery("SHOW STATUS LIKE 'Ssl_version'")) {
+            assertTrue(rs.next());
+            assertNotEquals("", rs.getString(1));
+        }
+    }
+
+    protected static void assertSecureConnection(Connection conn, String user) throws Exception {
+        assertSecureConnection(conn);
+        try (Statement stmt = conn.createStatement(); ResultSet rs = stmt.executeQuery("SELECT CURRENT_USER()")) {
+            assertTrue(rs.next());
+            assertEquals(user, rs.getString(1).split("@")[0]);
+        }
     }
 
     /*
@@ -1054,6 +1034,8 @@ public abstract class BaseTestCase extends TestCase {
     /**
      * Retrieve the current system time in milliseconds, using the nanosecond
      * time if possible.
+     * 
+     * @return current time in milliseconds
      */
     protected static final long currentTimeMillis() {
         try {
@@ -1069,24 +1051,24 @@ public abstract class BaseTestCase extends TestCase {
     }
 
     protected Connection getFailoverConnection(Properties props) throws SQLException {
-        return DriverManager.getConnection(getMasterSlaveUrl(), getHostFreePropertiesFromTestsuiteUrl(props));
+        return DriverManager.getConnection(getSourceReplicaUrl(), getHostFreePropertiesFromTestsuiteUrl(props));
     }
 
-    protected Connection getMasterSlaveReplicationConnection() throws SQLException {
-        return getMasterSlaveReplicationConnection(null);
+    protected Connection getSourceReplicaReplicationConnection() throws SQLException {
+        return getSourceReplicaReplicationConnection(null);
     }
 
-    protected Connection getMasterSlaveReplicationConnection(Properties props) throws SQLException {
-        String replicationUrl = getMasterSlaveUrl(ConnectionUrl.Type.REPLICATION_CONNECTION.getScheme());
+    protected Connection getSourceReplicaReplicationConnection(Properties props) throws SQLException {
+        String replicationUrl = getSourceReplicaUrl(ConnectionUrl.Type.REPLICATION_CONNECTION.getScheme());
         Connection replConn = new NonRegisteringDriver().connect(replicationUrl, getHostFreePropertiesFromTestsuiteUrl(props));
         return replConn;
     }
 
-    protected String getMasterSlaveUrl() throws SQLException {
-        return getMasterSlaveUrl(ConnectionUrl.Type.FAILOVER_CONNECTION.getScheme());
+    protected String getSourceReplicaUrl() throws SQLException {
+        return getSourceReplicaUrl(ConnectionUrl.Type.FAILOVER_CONNECTION.getScheme());
     }
 
-    protected String getMasterSlaveUrl(String protocol) throws SQLException {
+    protected String getSourceReplicaUrl(String protocol) throws SQLException {
         HostInfo hostInfo = mainConnectionUrl.getMainHost();
         String hostPortPair = TestUtils.encodePercent(hostInfo.getHostPortPair());
         return String.format("%s//%s,%s/", protocol, hostPortPair, hostPortPair);
@@ -1277,4 +1259,29 @@ public abstract class BaseTestCase extends TestCase {
         }
         return res;
     }
+
+    protected boolean supportsTimeZoneNames(Statement st) throws Exception {
+        ResultSet rs1 = st.executeQuery("SELECT COUNT(*) FROM mysql.time_zone_name");
+        return rs1.next() && rs1.getInt(1) > 0;
+    }
+
+    protected boolean supportsLoadLocalInfile(Statement st) throws Exception {
+        ResultSet rs1 = st.executeQuery("SHOW VARIABLES LIKE 'local_infile'");
+        return rs1.next() && "ON".equalsIgnoreCase(rs1.getString(2));
+    }
+
+    protected boolean supportsTestCertificates(Statement st) throws Exception {
+        ResultSet rs1 = st.executeQuery("SHOW VARIABLES LIKE 'ssl_ca'");
+        return rs1.next() && rs1.getString(2).contains("ssl-test-certs");
+    }
+
+    protected boolean supportsTestSha256PasswordKeys(Statement st) throws Exception {
+        ResultSet rs1 = st.executeQuery("SHOW VARIABLES LIKE 'sha256_password_public_key_path'");
+        return rs1.next() && rs1.getString(2).contains("ssl-test-certs");
+    }
+
+    protected boolean supportsTestCachingSha2PasswordKeys(Statement st) throws Exception {
+        ResultSet rs1 = st.executeQuery("SHOW VARIABLES LIKE 'caching_sha2_password_private_key_path'");
+        return rs1.next() && rs1.getString(2).contains("ssl-test-certs");
+    }
 }
