@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License, version 2.0, as published by the
@@ -29,6 +29,11 @@
 
 package testsuite.regression;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assumptions.assumeFalse;
+
 import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
@@ -43,6 +48,8 @@ import java.sql.Statement;
 import java.util.Properties;
 import java.util.concurrent.Callable;
 
+import org.junit.jupiter.api.Test;
+
 import com.mysql.cj.conf.PropertyKey;
 import com.mysql.cj.util.StringUtils;
 
@@ -52,28 +59,10 @@ import testsuite.BaseTestCase;
  * Tests fixes for BLOB handling.
  */
 public class BlobRegressionTest extends BaseTestCase {
-    /**
-     * Creates a new BlobRegressionTest.
-     * 
-     * @param name
-     *            name of the test to run
-     */
-    public BlobRegressionTest(String name) {
-        super(name);
-    }
-
-    /**
-     * Runs all test cases in this test suite
-     * 
-     * @param args
-     */
-    public static void main(String[] args) {
-        junit.textui.TestRunner.run(BlobRegressionTest.class);
-    }
-
     /**
      * @throws Exception
      */
+    @Test
     public void testBug2670() throws Exception {
         byte[] blobData = new byte[32];
 
@@ -99,38 +88,63 @@ public class BlobRegressionTest extends BaseTestCase {
 
         byte[] newBlobData = blob.getBytes(1L, (int) blob.length());
 
-        assertTrue("Blob changed length", blob.length() == blobData.length);
+        assertTrue(blob.length() == blobData.length, "Blob changed length");
 
-        assertTrue("New data inserted wrongly", ((newBlobData[3] == 2) && (newBlobData[4] == 2) && (newBlobData[5] == 2) && (newBlobData[6] == 2)));
+        assertTrue(((newBlobData[3] == 2) && (newBlobData[4] == 2) && (newBlobData[5] == 2) && (newBlobData[6] == 2)), "New data inserted wrongly");
 
         //
         // Test end-point insertion
         //
         blob.setBytes(32, new byte[] { 2, 2, 2, 2 });
 
-        assertTrue("Blob length should be 3 larger", blob.length() == (blobData.length + 3));
-
+        assertTrue(blob.length() == (blobData.length + 3), "Blob length should be 3 larger");
     }
 
     /**
      * http://bugs.mysql.com/bug.php?id=22891
      * 
      * @throws Exception
-     *             ...
      */
+    @Test
     public void testUpdateLongBlobGT16M() throws Exception {
-        byte[] blobData = new byte[18 * 1024 * 1024]; // 18M blob
+        this.rs = this.stmt.executeQuery("SHOW VARIABLES LIKE 'max_allowed_packet'");
+        this.rs.next();
+        long len = 4 + 1024 * 1024 * 36 + "UPDATE testUpdateLongBlob SET blobField=".length();
+        long defaultMaxAllowedPacket = this.rs.getInt(2);
+        boolean changeMaxAllowedPacket = defaultMaxAllowedPacket < len;
 
-        createTable("testUpdateLongBlob", "(blobField LONGBLOB)");
-        this.stmt.executeUpdate("INSERT INTO testUpdateLongBlob (blobField) VALUES (NULL)");
-
-        this.pstmt = this.conn.prepareStatement("UPDATE testUpdateLongBlob SET blobField=?");
-        this.pstmt.setBytes(1, blobData);
+        Connection con1 = null;
+        Connection con2 = null;
         try {
-            this.pstmt.executeUpdate();
-        } catch (SQLException sqlEx) {
-            if (sqlEx.getMessage().indexOf("max_allowed_packet") != -1) {
-                fail("You need to increase max_allowed_packet to at least 18M before running this test!");
+            if (changeMaxAllowedPacket) {
+                this.stmt.executeUpdate("SET GLOBAL max_allowed_packet=" + 1024 * 1024 * 37);
+            }
+            byte[] blobData = new byte[18 * 1024 * 1024]; // 18M blob
+
+            createTable("testUpdateLongBlob", "(blobField LONGBLOB)");
+            this.stmt.executeUpdate("INSERT INTO testUpdateLongBlob (blobField) VALUES (NULL)");
+
+            Properties props = new Properties();
+            props.setProperty(PropertyKey.useSSL.getKeyName(), "false");
+            props.setProperty(PropertyKey.allowPublicKeyRetrieval.getKeyName(), "true");
+            con1 = getConnectionWithProps(props);
+            this.pstmt = con1.prepareStatement("UPDATE testUpdateLongBlob SET blobField=?");
+            this.pstmt.setBytes(1, blobData);
+            try {
+                this.pstmt.executeUpdate();
+            } catch (SQLException sqlEx) {
+                assertTrue(sqlEx.getMessage().indexOf("max_allowed_packet") == -1,
+                        "You need to increase max_allowed_packet to at least 18M before running this test!");
+            }
+        } finally {
+            if (changeMaxAllowedPacket) {
+                this.stmt.executeUpdate("SET GLOBAL max_allowed_packet=" + defaultMaxAllowedPacket);
+            }
+            if (con1 != null) {
+                con1.close();
+            }
+            if (con2 != null) {
+                con2.close();
             }
         }
     }
@@ -138,6 +152,7 @@ public class BlobRegressionTest extends BaseTestCase {
     /**
      * @throws Exception
      */
+    @Test
     public void testUpdatableBlobsWithCharsets() throws Exception {
         byte[] smallBlob = new byte[32];
 
@@ -173,13 +188,13 @@ public class BlobRegressionTest extends BaseTestCase {
             byte origValue = smallBlob[i];
             byte newValue = updatedBlob[i];
 
-            assertTrue("Original byte at position " + i + ", " + origValue + " != new value, " + newValue, origValue == newValue);
+            assertTrue(origValue == newValue, "Original byte at position " + i + ", " + origValue + " != new value, " + newValue);
         }
 
     }
 
+    @Test
     public void testBug5490() throws Exception {
-
         createTable("testBug5490", "(pk INT NOT NULL PRIMARY KEY, blobField BLOB)");
         String sql = "insert into testBug5490 values(?,?)";
 
@@ -201,20 +216,20 @@ public class BlobRegressionTest extends BaseTestCase {
         byte[] returned = this.rs.getBytes(1);
 
         assertEquals(blobFileSize, returned.length);
-
     }
 
     /**
-     * Tests BUG#8096 where emulated locators corrupt binary data when using
-     * server-side prepared statements.
+     * Tests BUG#8096 where emulated locators corrupt binary data when using server-side prepared statements.
      * 
      * @throws Exception
-     *             if the test fails.
      */
+    @Test
     public void testBug8096() throws Exception {
         int dataSize = 256;
 
         Properties props = new Properties();
+        props.setProperty(PropertyKey.useSSL.getKeyName(), "false");
+        props.setProperty(PropertyKey.allowPublicKeyRetrieval.getKeyName(), "true");
         props.setProperty(PropertyKey.emulateLocators.getKeyName(), "true");
         Connection locatorConn = getConnectionWithProps(props);
 
@@ -267,18 +282,17 @@ public class BlobRegressionTest extends BaseTestCase {
             // Will print out all of the values that don't match.
             // All negative values will instead be replaced with 63.
             if (result[i] != testData[i]) {
-                assertEquals("At position " + i, testData[i], result[i]);
+                assertEquals(testData[i], result[i], "At position " + i);
             }
         }
     }
 
     /**
-     * Tests fix for BUG#9040 - PreparedStatement.addBatch() doesn't work with
-     * server-side prepared statements and streaming BINARY data.
+     * Tests fix for BUG#9040 - PreparedStatement.addBatch() doesn't work with server-side prepared statements and streaming BINARY data.
      * 
      * @throws Exception
-     *             if the test fails.
      */
+    @Test
     public void testBug9040() throws Exception {
 
         createTable("testBug9040", "(primary_key int not null primary key, data mediumblob)");
@@ -298,9 +312,9 @@ public class BlobRegressionTest extends BaseTestCase {
         this.pstmt.addBatch();
 
         this.pstmt.executeBatch();
-
     }
 
+    @Test
     public void testBug10850() throws Exception {
         String tableName = "testBug10850";
 
@@ -321,9 +335,9 @@ public class BlobRegressionTest extends BaseTestCase {
 
         assertEquals("0", getSingleIndexedValueWithQuery(1, "SELECT LENGTH(field1) FROM " + tableName).toString());
         this.stmt.executeUpdate("TRUNCATE TABLE " + tableName);
-
     }
 
+    @Test
     public void testBug34677() throws Exception {
         createTable("testBug34677", "(field1 BLOB)");
         this.stmt.executeUpdate("INSERT INTO testBug34677 VALUES ('abc')");
@@ -334,15 +348,14 @@ public class BlobRegressionTest extends BaseTestCase {
         blob.truncate(0L);
         assertEquals(0, blob.length());
         assertEquals(-1, blob.getBinaryStream().read());
-
     }
 
     /**
      * Tests fix for BUG#20453671 - CLOB.POSITION() API CALL WITH CLOB INPUT RETURNS EXCEPTION
      * 
      * @throws Exception
-     *             if the test fails.
      */
+    @Test
     public void testBug20453671() throws Exception {
         this.rs = this.stmt.executeQuery("select 'abcd', 'a', 'b', 'c', 'd', 'e'");
         this.rs.next();
@@ -374,8 +387,8 @@ public class BlobRegressionTest extends BaseTestCase {
      * server-side prepared statements and streaming BINARY data.
      * 
      * @throws Exception
-     *             if the test fails.
      */
+    @Test
     public void testBug20453712() throws Exception {
         final String s1 = "NewClobData";
         this.rs = this.stmt.executeQuery("select 'a'");
@@ -418,27 +431,63 @@ public class BlobRegressionTest extends BaseTestCase {
      * 
      * @throws Exception
      */
+    @Test
     public void testBug23535571() throws Exception {
 
+        this.rs = this.stmt.executeQuery("SHOW VARIABLES LIKE 'max_allowed_packet'");
+        this.rs.next();
+        long len = 4 + 1024 * 1024 * 36 + "UPDATE testBug23535571 SET blobField=".length();
+        long defaultMaxAllowedPacket = this.rs.getInt(2);
+        boolean changeMaxAllowedPacket = defaultMaxAllowedPacket < len;
+
+        if (!versionMeetsMinimum(5, 7)) {
+            this.rs = this.stmt.executeQuery("SHOW VARIABLES LIKE 'innodb_log_file_size'");
+            this.rs.next();
+            long defaultInnodbLogFileSize = this.rs.getInt(2);
+            assumeFalse(defaultInnodbLogFileSize < 1024 * 1024 * 36 * 10, "This test requires innodb_log_file_size > " + (1024 * 1024 * 36 * 10));
+        }
+
         createTable("testBug23535571", "(blobField LONGBLOB)");
         this.stmt.executeUpdate("INSERT INTO testBug23535571 (blobField) VALUES (NULL)");
 
-        // Insert 1 record with 18M data
-        byte[] blobData = new byte[18 * 1024 * 1024];
-        this.pstmt = this.conn.prepareStatement("UPDATE testBug23535571 SET blobField=?");
-        this.pstmt.setBytes(1, blobData);
-        this.pstmt.executeUpdate();
+        Connection con1 = null;
+        Connection con2 = null;
+        try {
+            if (changeMaxAllowedPacket) {
+                this.stmt.executeUpdate("SET GLOBAL max_allowed_packet=" + 1024 * 1024 * 37);
+            }
 
-        Properties props = new Properties();
-        props.setProperty(PropertyKey.enablePacketDebug.getKeyName(), "true");
-        Connection con = getConnectionWithProps(props);
+            Properties props = new Properties();
+            props.setProperty(PropertyKey.useSSL.getKeyName(), "false");
+            props.setProperty(PropertyKey.allowPublicKeyRetrieval.getKeyName(), "true");
+            con1 = getConnectionWithProps(props);
 
-        for (int i = 0; i < 100; i++) {
-            this.pstmt = con.prepareStatement("select * from testBug23535571");
-            this.rs = this.pstmt.executeQuery();
-            this.rs.close();
-            this.pstmt.close();
-            Thread.sleep(100);
+            // Insert 1 record with 18M data
+            byte[] blobData = new byte[18 * 1024 * 1024];
+            this.pstmt = con1.prepareStatement("UPDATE testBug23535571 SET blobField=?");
+            this.pstmt.setBytes(1, blobData);
+            this.pstmt.executeUpdate();
+
+            props.setProperty(PropertyKey.enablePacketDebug.getKeyName(), "true");
+            con2 = getConnectionWithProps(props);
+
+            for (int i = 0; i < 100; i++) {
+                this.pstmt = con2.prepareStatement("select * from testBug23535571");
+                this.rs = this.pstmt.executeQuery();
+                this.rs.close();
+                this.pstmt.close();
+                Thread.sleep(100);
+            }
+        } finally {
+            if (changeMaxAllowedPacket) {
+                this.stmt.executeUpdate("SET GLOBAL max_allowed_packet=" + defaultMaxAllowedPacket);
+            }
+            if (con1 != null) {
+                con1.close();
+            }
+            if (con2 != null) {
+                con2.close();
+            }
         }
 
     }
@@ -447,15 +496,17 @@ public class BlobRegressionTest extends BaseTestCase {
      * Tests BUG#95210, ClassCastException in BlobFromLocator when connecting as jdbc:mysql:replication.
      * 
      * @throws Exception
-     *             if the test fails.
      */
+    @Test
     public void testBug95210() throws Exception {
         createTable("testBug95210", "(ID VARCHAR(10) PRIMARY KEY, DATA LONGBLOB)");
         this.stmt.executeUpdate("INSERT INTO testBug95210 (ID, DATA) VALUES (1, '111')");
 
         Properties props = new Properties();
+        props.setProperty(PropertyKey.useSSL.getKeyName(), "false"); // testsuite is built upon non-SSL default connection
+        props.setProperty(PropertyKey.allowPublicKeyRetrieval.getKeyName(), "true");
         props.setProperty(PropertyKey.emulateLocators.getKeyName(), "true");
-        Connection locatorConn = getMasterSlaveReplicationConnection(props);
+        Connection locatorConn = getSourceReplicaReplicationConnection(props);
 
         this.rs = locatorConn.createStatement().executeQuery("SELECT ID, 'DATA' AS BLOB_DATA from testBug95210");
         assertTrue(this.rs.next());
