@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License, version 2.0, as published by the
@@ -29,16 +29,22 @@
 
 package testsuite.x.devapi;
 
-import static org.junit.Assert.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 
 import java.lang.reflect.Field;
+import java.net.InetAddress;
+import java.net.UnknownHostException;
 import java.sql.SQLException;
+import java.util.Arrays;
 
 import com.mysql.cj.MysqlxSession;
 import com.mysql.cj.conf.PropertyKey;
 import com.mysql.cj.exceptions.MysqlErrorNumbers;
 import com.mysql.cj.protocol.x.XProtocolError;
+import com.mysql.cj.util.StringUtils;
+import com.mysql.cj.xdevapi.DocResult;
 import com.mysql.cj.xdevapi.PreparableStatement;
+import com.mysql.cj.xdevapi.Row;
 import com.mysql.cj.xdevapi.Schema;
 import com.mysql.cj.xdevapi.Session;
 import com.mysql.cj.xdevapi.SessionImpl;
@@ -60,6 +66,8 @@ public class DevApiBaseTestCase extends InternalXBaseTestCase {
     String dbCharset;
     String dbCollation;
 
+    private Boolean mysqlRunningLocally = null;
+
     public boolean setupTestSession() {
         if (this.isSetForXTests) {
             this.session = new SessionImpl(this.testHostInfo);
@@ -68,6 +76,32 @@ public class DevApiBaseTestCase extends InternalXBaseTestCase {
             this.dbCharset = rs.fetchOne().getString(1);
             rs = this.session.sql("SHOW VARIABLES LIKE 'collation_database'").execute();
             this.dbCollation = rs.fetchOne().getString(1);
+
+            // ensure max_connections value is enough to run tests
+            int maxConnections = 0;
+            int mysqlxMaxConnections = 0;
+
+            rs = this.session.sql("SHOW VARIABLES LIKE '%max_connections'").execute();
+            Row r = rs.fetchOne();
+            if (r.getString(0).contains("mysqlx")) {
+                mysqlxMaxConnections = r.getInt(1);
+                maxConnections = rs.fetchOne().getInt(1);
+            } else {
+                maxConnections = r.getInt(1);
+                mysqlxMaxConnections = rs.fetchOne().getInt(1);
+            }
+
+            rs = this.session.sql("show status like 'threads_connected'").execute();
+            int usedConnections = rs.fetchOne().getInt(1);
+
+            if (maxConnections - usedConnections < 200) {
+                maxConnections += 200;
+                this.session.sql("SET GLOBAL max_connections=" + maxConnections).execute();
+                if (mysqlxMaxConnections < maxConnections) {
+                    this.session.sql("SET GLOBAL mysqlx_max_connections=" + maxConnections).execute();
+                }
+            }
+
             return true;
         }
         return false;
@@ -126,6 +160,23 @@ public class DevApiBaseTestCase extends InternalXBaseTestCase {
         return res.fetchOne().getString(1).indexOf('\\') != -1;
     }
 
+    protected boolean isMysqlRunningLocally() {
+        if (this.mysqlRunningLocally != null) {
+            return this.mysqlRunningLocally;
+        }
+        try {
+            String clientHostname = InetAddress.getLocalHost().getHostName();
+
+            SqlResult res = this.session.sql("SHOW VARIABLES LIKE 'hostname'").execute();
+            String serverHostname = res.fetchOne().getString(1);
+
+            this.mysqlRunningLocally = clientHostname.equalsIgnoreCase(serverHostname);
+        } catch (UnknownHostException e) {
+            this.mysqlRunningLocally = false;
+        }
+        return this.mysqlRunningLocally;
+    }
+
     protected int getThreadId(Session sess) {
         return sess.sql("SELECT thread_id FROM performance_schema.threads WHERE processlist_id = connection_id()").execute().fetchOne().getInt(0);
     }
@@ -166,6 +217,15 @@ public class DevApiBaseTestCase extends InternalXBaseTestCase {
         return -1;
     }
 
+    protected boolean supportsTestCertificates(Session sess) {
+        SqlResult res = sess.sql("SELECT @@mysqlx_ssl_ca, @@ssl_ca").execute();
+        if (res.hasNext()) {
+            Row r = res.next();
+            return (StringUtils.isNullOrEmpty(r.getString(1)) && r.getString(2).contains("ssl-test-certs") || r.getString(1).contains("ssl-test-certs"));
+        }
+        return false;
+    }
+
     int getPreparedStatementId(PreparableStatement<?> stmt) {
         try {
             Field prepStmtId = PreparableStatement.class.getDeclaredField("preparedStatementId");
@@ -203,4 +263,40 @@ public class DevApiBaseTestCase extends InternalXBaseTestCase {
         } while (psCount != 0 && --countdown > 0);
         assertEquals(expectedCount, psCount);
     }
+
+    protected static void assertNonSecureSession(Session sess) {
+        assertSessionStatusEquals(sess, "mysqlx_ssl_cipher", "");
+    }
+
+    protected static void assertSecureSession(Session sess) {
+        assertSessionStatusNotEquals(sess, "mysqlx_ssl_cipher", "");
+    }
+
+    protected static void assertSecureSession(Session sess, String user) {
+        assertSecureSession(sess);
+        SqlResult res = sess.sql("SELECT CURRENT_USER()").execute();
+        assertEquals(user, res.fetchOne().getString(0).split("@")[0]);
+    }
+
+    public String buildString(int length, char charToFill) {
+        if (length > 0) {
+            char[] array = new char[length];
+            Arrays.fill(array, charToFill);
+            return new String(array);
+        }
+        return "";
+    }
+
+    public int count_data(DocResult docs1) {
+        int recCnt = 0;
+        while (true) {
+            try {
+                docs1.next();
+                recCnt++;
+            } catch (java.util.NoSuchElementException sqlEx) {
+                break;
+            }
+        }
+        return recCnt;
+    }
 }
